package core.notevault.ui.gallery;

import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.navigation.NavController;
import androidx.navigation.NavDirections;
import androidx.navigation.Navigation;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import core.notevault.R;
import core.notevault.data.Concert;
import core.notevault.data.MusicDatabase;
import core.notevault.data.MusicNote;
import core.notevault.data.MusicNoteDAO;
import core.notevault.databinding.FragmentGalleryBinding;
import core.notevault.ui.gallery.editor.ConcertEditorDialog;

import java.util.List;

public class GalleryFragment extends Fragment {

    private FragmentGalleryBinding binding;
    private ConcertAdapter concertAdapter;
    private GalleryViewModel galleryViewModel;

    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState) {
        galleryViewModel =
                new ViewModelProvider(this).get(GalleryViewModel.class);

        binding = FragmentGalleryBinding.inflate(inflater, container, false);
        View root = binding.getRoot();

        RecyclerView recyclerView = root.findViewById(R.id.concert_recycler_view);
        concertAdapter = new ConcertAdapter(new ConcertAdapter.OnConcertClickListener() {
            @Override
            public void onDeleteConcert(Concert concert) {
                deleteConcert(concert);
            }

            @Override
            public void onOpenConcertEditor(Concert concert) {
                editConcert(concert);
            }

            @Override
            public void onConcertClick(Concert concert) {
                Log.d("GalleryFragment", "Clicked on Concert: " + concert.getTitle());
                Bundle bundle = new Bundle();
                bundle.putInt("concertID", concert.getId());
                bundle.putString("concertTitle", concert.getTitle());
                bundle.putString("concertDate", concert.getConcertDate());

                NavController navController = Navigation.findNavController(getActivity(), R.id.nav_host_fragment_content_main);
                navController.navigate(R.id.concertDetailFragment, bundle);
            }
        });
        recyclerView.setAdapter(concertAdapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));

        DividerItemDecoration dividerItemDecoration = new DividerItemDecoration(recyclerView.getContext(), LinearLayoutManager.VERTICAL);
        recyclerView.addItemDecoration(dividerItemDecoration);


        FloatingActionButton addConcertBtn = root.findViewById(R.id.add_concert_btn);
        addConcertBtn.setOnClickListener(v -> onCreateNewConcert());

        galleryViewModel.getConcerts().observe(getViewLifecycleOwner(), newConcerts -> {
            concertAdapter.updateConcerts(newConcerts);
        });

        new LoadConcerts().execute();
        return root;
    }

    public void deleteConcert(Concert concert) {
        Log.d("GalleryFragment", "Delete Concert");
        new Thread(() -> {
            MusicDatabase musicDatabase = MusicDatabase.getDatabase(this.getContext());
            MusicNoteDAO musicNoteDAO = musicDatabase.musicNoteDao();
            musicNoteDAO.deleteConcert(concert);

            new Handler(Looper.getMainLooper()).post(() -> {
                galleryViewModel.deleteConcert(concert);
                Log.d("GalleryFragment", "Concert deleted successfully and ViewModel updated");
            });
        }).start();
    }

    public void editConcert(Concert concert) {
        ConcertEditorDialog concertEditorDialog = new ConcertEditorDialog();

        Bundle args = new Bundle();
        args.putString("concert_title", concert.getTitle());
        args.putString("concert_date", concert.getConcertDate());
        args.putInt("concertID", concert.getId());

        concertEditorDialog.setArguments(args);

        concertEditorDialog.show(getParentFragmentManager(), ConcertEditorDialog.TAG);
    }

    public void updateConcert(Concert concert) {
        galleryViewModel.updateConcert(concert);
    }

    private class LoadConcerts extends AsyncTask<Void, Void, List<Concert>> {

        @Override
        protected List<Concert> doInBackground(Void... voids) {
            MusicDatabase db = MusicDatabase.getDatabase(getContext());
            MusicNoteDAO musicNoteDAO = db.musicNoteDao();
            List<Concert> concerts =  musicNoteDAO.getAllConcerts();
            Log.d("GalleryFragment", "Fetching concerts: " + concerts.size());
            return concerts;

        }

        @Override
        protected void onPostExecute(List<Concert> concerts) {
            Log.d("GalleryFragment", "Concerts size in onPostExecute: " + concerts.size());
            // concertAdapter.updateConcerts(concerts); // Aktualisiere den Adapter mit den Titeln
            galleryViewModel.setConcerts(concerts);
        }
    }

    public void onCreateNewConcert() {
        ConcertEditorDialog concertEditorDialog = new ConcertEditorDialog();
        concertEditorDialog.show(getParentFragmentManager(), ConcertEditorDialog.TAG);
    }

    public void addConcert(Concert concert) {
        this.galleryViewModel.addConcert(concert);
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
    }
}